using System;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using GHI.Pins;

using Microtoolskit.Hardware.Displays;

namespace PANDA_3_Anemometre
{ // Titre : Mesure de la vitesse du vent avec un kit Weather Sensor Assembly p/n 80422 https://goo.gl/Pdbzt9
    public class Program
    {
        // Compteurs: mmorisent le nombre de tours de l'anmomtre pendant un temps t = sample
        static UInt32 Counter = 0, Counter_1 = 0;
        public static void Main()
        {
            // variables
            UInt32 RazCount = 0;
            var delay = 0;    // Attente avant chantillonnage de la vitesse du vent
            var sample = 2;   // Priode d'chantillonnage en seconde,  rgler en fonction
            // de l'erreur accepte sur la mesure 
            // Exemple:  sample = 20*Tsignal pour une erreur de 5%

            // LCD  connecter sur la sortie O5 de la carte Tinkerkit           
            // Documentation de la classe ELCD162 : https://github.com/WebGE?tab=repositories
            // Cration et initialisation d'un objet afficheur srie ELCD-162 (par dfaut: COM2, 19200, None, 8, 1)
            var Lcd = new ELCD162(); Lcd.Init(); Lcd.ClearScreen();

            // Acquisition du signal TOR de l'anmomtre sur l'entre I0 de la carte Thinkerkit
            var signal = new InterruptPort(FEZPandaIII.Gpio.A0, true, Port.ResistorMode.Disabled, Port.InterruptMode.InterruptEdgeHigh);

            // Inscription de l'interruption
            signal.OnInterrupt += new NativeEventHandler(measureFreq);

            // Timer utilis pour la remise  zro du compteur aprs chaque chantillonnage de la vitesse du vent
            signal.DisableInterrupt();
            var timer = new Timer(new TimerCallback(SaveandRazCounter), RazCount, delay, (int)(1000 * sample));
            Counter = 0; signal.EnableInterrupt();

            while (true)
            {// la mesure de la frquence de rotation de l'anmomtre est dsactive pendant l'affichage
                signal.DisableInterrupt(); // Dsactivation de l'interruption pour ne pas perturber l'affichage
                var speed = 2.4 * (Counter_1 / sample);
                Lcd.ClearScreen(); Lcd.PutString("Vvent=" + speed.ToString("F1") + "km/h"); // Affichage de la frquence
                Counter = 0;
                signal.EnableInterrupt(); // Autorisation de l'interruption
                Thread.Sleep(5000 * sample);
            }
        }

        // ---------------------------------------------------------------------------------------------
        // Gestionnaires d'interruption
        // ---------------------------------------------------------------------------------------------
        // Raz compteur
        static void SaveandRazCounter(object RazCount)
        {
            Counter_1 = Counter;
            Counter = (uint)RazCount;
        }
        // ---------------------------------------------------------------------------------------------
        // Compteur
        static void measureFreq(uint port, uint state, DateTime time)
        {
            Counter++;
        }
        // ---------------------------------------------------------------------------------------------
    }
}
